#!/usr/bin/perl

use strict;
use Data::Dumper;
require("libmreg.pl");

################################################################################
# C O N F I G U R A T I O N   S E C T I O N                                    #
################################################################################

## Edit these values for login to the API
my $api_login = "USERNAME";
my $api_password = "PASSWORD";

# optional; needed if there are no domain repositories assigned to the user account
my $trigger = "";

## Add your API-IP or -Hostname
my $api_host = "93.190.235.100";
my $api_config = { 'socket' => 'http://' . $api_host . '/api/call.cgi?s_login=' . $api_login . '&s_pw=' . $api_password};


###############################################################################
## P R O G R A M M   S E C T I O N                                            #
## please do not edit beyond this line                                        #
###############################################################################

my $error_msg;
my $warning_msg;
my $ok_msg;

## QueryDomainRepositoryList
my $command_repository_list = { 
	'COMMAND' => 'QueryDomainRepositoryList',
};

if ($trigger) {
	$command_repository_list->{'TRIGGER'} = $trigger;
}

debug(2,"COMMAND: " . Dumper($command_repository_list));
my $response = mreg_call($command_repository_list, $api_config);
if ($$response{'CODE'} != 200) {
	error("Command QueryDomainRepositoryList failed: CODE: " . $$response{'CODE'} . " DESCRIPTION: " . $$response{'DESCRIPTION'});
}

debug(1,"DEBUG: " . Dumper($response));
foreach my $repository( @{$response->{'PROPERTY'}->{'REPOSITORY'}} ) {
	my $status_account_possible = 0;
	debug(2,"REPOSITORY: " . $repository);

	## StatusDomainRepository
	my $command_status_repository = {
		'COMMAND' => 'StatusDomainRepository',
		'REPOSITORY' => $repository
	};
	
	if ($trigger) {
		$command_status_repository->{'TRIGGER'} = $trigger;
	}
	debug(1, "Call Command");
	$response = mreg_call($command_status_repository, $api_config);
	if ($$response{'CODE'} != 200) {
		error("Command StatusDomainRepository for repository " . $repository . " failed: CODE: " . $$response{'CODE'} . " DESCRIPTION: " . $$response{'DESCRIPTION'});
	}
	else {
		## TRADES
		debug(1,"DEBUG: " . Dumper($response));
		my $max_trades = $response->{'PROPERTY'}->{'REPOSITORY_TRADES_MAX'}[0];
		debug(2, "MAX TRADES: " . $max_trades);
		my $current_trades = $response->{'PROPERTY'}->{'SUM_TRADES'}[0];
		debug(2, "TRADES: " . $current_trades);
	
		if (!($max_trades)) {
			error("REPOSITORY_TRADES_MAX for repository " . $repository . " not found in db");
		}

		if ($max_trades <= $current_trades) {
			debug(3, "ERROR: Number of trades => threshold");
			$warning_msg .= $repository . ": Number of trades => threshold (" . $max_trades . " <= " . $current_trades . ") / ";
		}

		## OUTGOING TRANSFERS

		my $max_outgoing_transfers = $response->{'PROPERTY'}->{'REPOSITORY_TRANSFERS_OUTGOING_MAX'}[0];
		debug(2, "MAX OUTGOING TRANSFERS: " . $max_outgoing_transfers);
		my $current_outgoing_transfers = $response->{'PROPERTY'}->{'SUM_OUTGOING_TRANSFERS'}[0];
		debug(2, "CURRENT OUTGOING TRANSFERS: " . $current_outgoing_transfers);

		if (!($max_outgoing_transfers)) {
			error("REPOSITORY_TRANSFERS_OUTGOING_MAX for repository " . $repository . " not found in db");
		}

		if ($max_outgoing_transfers <= $current_outgoing_transfers) {
			debug(3, "ERROR: Number of outgoing transfers => threshold");
			$warning_msg .= $repository . ": Number of outgoing transfers => threshold (" . $max_outgoing_transfers . " <= " . $current_outgoing_transfers . ") / ";
		}
		
		$status_account_possible = $response->{'PROPERTY'}->{'STATUS_ACCOUNT_AVAILABLE'}[0]
			if (length($response->{'PROPERTY'}->{'STATUS_ACCOUNT_AVAILABLE'}[0]));
		debug(4, "STATUS: " . $status_account_possible . " / " . $repository);
	}

	if ($status_account_possible eq "1") {
		## StatusDomainAccount
		my $command_status_account = {
			'COMMAND' => 'StatusDomainAccount',
			'REPOSITORY' => $repository
		};
		
		$command_status_account->{'TRIGGER'} = $trigger
			if ($trigger):
		
		$response = mreg_call($command_status_account, $api_config);
		if ($$response{'CODE'} != 200) {
			error("Command StatusDomainAccount for repository " . $repository . " failed: CODE: " . $$response{'CODE'} . " DESCRIPTION: " . $$response{'DESCRIPTION'});
		}
		else {
			debug(1,"DEBUG: " . Dumper($response));
			
			if (!($response->{'PROPERTY'}->{'ZONE'}[0])) {
				my $max_credit_amount = int(($response->{'PROPERTY'}->{'ACCOUNTAMOUNTMAX'}[0]))
					if (length($response->{'PROPERTY'}->{'ACCOUNTAMOUNTMAX'}[0]));
				my $min_credit_amount = int(($response->{'PROPERTY'}->{'ACCOUNTAMOUNTMIN'}[0]))
					if (length($response->{'PROPERTY'}->{'ACCOUNTAMOUNTMIN'}[0]));
				my $current_credit = $response->{'PROPERTY'}->{'CREDIT'}[0];

				if (!($max_credit_amount)) {
					error("Cannot find 'ACCOUNTAMOUNTMAX' for repository " . $repository);
				}
				if (!($min_credit_amount)) {
					error("Cannot find 'ACCOUNTAMOUNTMIN' for repository " . $repository);
				}
				if ( (!length($current_credit) > 0) || ($current_credit !~ /^[\-]*[0-9]+$/) ) {
					warning("Cannot find 'CREDIT' for repository " . $repository);
				}
				$min_credit_amount = int($min_credit_amount / 2);
	
				if ($current_credit < $min_credit_amount) {
					$error_msg .= $repository . ": Credit is < " . $min_credit_amount . " (" . $current_credit . ") / ";
				}
				elsif ( ($max_credit_amount > 0) && ($current_credit > $max_credit_amount) ) {
					$error_msg .= $repository . ": Credit is > " . $max_credit_amount . " (" . $current_credit . ") / ";
				}
				else {
					## OK
				}
			}
			else {
				foreach my $value ( @{$response->{'PROPERTY'}{'ZONE'}}) {
					my $max_credit_amount = int($response->{'PROPERTY'}->{"ACCOUNTAMOUNTMAX_" . $value}[0])
						if (length($response->{'PROPERTY'}->{"ACCOUNTAMOUNTMAX_" . $value}[0]));
					my $min_credit_amount = int($response->{'PROPERTY'}->{"ACCOUNTAMOUNTMIN_" . $value}[0])
						if (length($response->{'PROPERTY'}->{"ACCOUNTAMOUNTMIN_" . $value}[0]));
					my $current_credit = $response->{'PROPERTY'}->{"CREDIT_" . $value}[0];
	
					if (!($max_credit_amount)) {
						error("Cannot find 'ACCOUNTAMOUNTMAX_" . $value . "' for repository " . $repository . " and TLD " . $value);
					}
					if (!($min_credit_amount)) {
						error("Cannot find 'ACCOUNTAMOUNTMIN_" . $value . "' for repository " . $repository . " and TLD " . $value);
					}
					if ( (!length($current_credit) > 0) || ($current_credit !~ /^[\-]*[0-9]+$/) ) { 
						error("Cannot find 'CREDIT_" . $value . "' for repository " . $repository . " and TLD " . $value);
					}
	
					if ($current_credit < $min_credit_amount) {
						$error_msg .= $repository . ": Credit is < " . $min_credit_amount . " (" . $current_credit . ") / ";
					}
					elsif ( ($max_credit_amount > 0) && ($current_credit > $max_credit_amount) ) {
						$error_msg .= $repository . ": Credit is > " . $max_credit_amount . " (" . $current_credit . ") / ";
					}
					else {
						## OK
					}
				}
			}
		}
	}
}

if (!($error_msg)) {
	print "OK: All repositories are ok\n";
	exit 0;
}
else { 
	print "ERROR: ". $error_msg . "\n";
	exit 2;
}


## SUB
######

sub error
{
	my ($msg) = @_;
	print "ERROR: " . $msg . "\n";
	exit 2;
}

sub warning
{
	my ($msg) = @_;
	print "WARNING: " . $msg . "\n";
	exit 1;
}

sub debug
{
	my $level = shift;
	my ($msg) = shift;

	## DEBUG-LEVEL
	# 0 = DEBUG, 1 = INFO, 2 = WARNING, 3 = ERROR, 4 = FATAL, 5 = OFF
	my $debug_level = 5;

	if ($level >= $debug_level) {
		print $msg . "\n";
	}
}
